package org.openapiusage;

import org.alx.openapi.v1.ApiClient;
import org.alx.openapi.v1.ApiException;
import org.alx.openapi.v1.api.OpenApiApi;
import org.alx.openapi.v1.auth.ApiKeyAuth;
import org.alx.openapi.v1.model.*;

import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.time.LocalDateTime;
import java.util.List;

import static java.util.stream.Collectors.joining;
import static org.alx.openapi.v1.model.ApiExperimentResult.ValidationStatusEnum.HAS_ERRORS;

public class ApiTutorialPhase3 {

    public static final String ALX_SERVER_ADDRESS = "https://your-server-address-here.com/api/v1";
    public static final String ALX_API_KEY = "your-api-key-here";

    public static final String PROJECT_NAME = "API_tutorial";

    public static final String IMPORTING_FILENAME = "C:/api_tutorial/Example1.xlsx";
    public static final String RESULT_EXPORT_FILENAME = "C:/api_tutorial/%s dashboard data.xlsx";

    public static void main(String[] args) throws ApiException {
        System.out.println("API Tutorial welcomes you");

        // 1. Create connection to ALX Server (api client)
        var apiClient = new ApiClient();
        apiClient.setBasePath(ALX_SERVER_ADDRESS);

        var apiKeyAuth = (ApiKeyAuth) apiClient.getAuthentication("ApiKey");
        apiKeyAuth.setApiKey(ALX_API_KEY);

        // This object will be used to execute all API methods
        var api = new OpenApiApi(apiClient);
        apiClient.setReadTimeout(50000);

        // 2. Connect to the existing project
        var projectData = api.findAndOpenProjectByName(true, PROJECT_NAME);
        System.out.println(projectData);

        // 3. Import scenario
        var scenarioData = api.importScenario(projectData.getId(), new File(IMPORTING_FILENAME));

        // 4. Get Simulation experiment from the scenario
        ApiSimulationRunConfigurationWrapper simulationExperiment = (ApiSimulationRunConfigurationWrapper)
                api.getExperiment(ApiBasicRun.TypeEnum.SIMULATION.getValue(), scenarioData.getId());
        var simulationRunConfiguration = simulationExperiment.getConfiguration();

        // 5. Define new stop date for Simulation experiment
        var newStopDate = LocalDateTime.of(2023, 6, 1, 0, 0, 0);
        System.out.println("Old ending date: " + simulationRunConfiguration.getStopDate() + ",\nNew ending date: " + newStopDate);

        ApiIdNameObject customEndType = getCustomEndTypeProperty(api, simulationRunConfiguration);
        simulationRunConfiguration.setTimeTypeEnd(customEndType);
        simulationRunConfiguration.setStopDate(String.valueOf(newStopDate));

        // 6. Apply all changes sending them to ALX
        simulationExperiment = (ApiSimulationRunConfigurationWrapper) api.updateExperimentRunConfiguration(simulationExperiment);
        // Check new experiment settings
        simulationRunConfiguration = simulationExperiment.getConfiguration();
        System.out.println("Applied new ending date: " + simulationRunConfiguration.getStopDate());

        // 7. Run Simulation experiment
        System.out.println("Simulation experiment started...");
        ApiExperimentResult simExperimentResult = api.runExperimentSynchronously(simulationRunConfiguration.getId());
        System.out.println("Simulation experiment completed...");

        if (simExperimentResult.getValidationStatus() == HAS_ERRORS) {
            System.err.println(toStringValidationErrors(api, scenarioData));
            return;
        }

        // 8. Get experiment results and dashboard and trace contents
        List<ApiDashboardPage> experimentDashboardPages = api.getExperimentDashboardPages(simExperimentResult.getExperimentResultId());
        for (var dashboardPage : experimentDashboardPages) {
            System.out.println(dashboardPage.getName());
            for(var dashboardChartData: dashboardPage.getCharts()) {
                System.out.println("\t" + dashboardChartData.getName());
            }
        }

        // 9. Find the required dashboard page and get its data.
        var page = api.getExperimentDashboardPage(
                simExperimentResult.getExperimentResultId(), true, "Profit and Loss Statement");
        var path = String.format(RESULT_EXPORT_FILENAME, page.getName());
        var exportResult = api.exportDashboardPage(simExperimentResult.getExperimentResultId(), page.getId());

        // 10. Save the data, then open the MS Excel file
        var outputFile = new File(path);
        try {
            Files.write(outputFile.toPath(), exportResult);
            Desktop.getDesktop().open(outputFile);
            System.out.println("Exported page " + page.getName() + "\nPage data exported to " + outputFile.getName());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        // 11. Remove scenario and Close project
//        api.removeScenarioById(scenarioData.getId());
//        var closeProjectOpenResponse = api.closeProject();
//        System.out.println(closeProjectOpenResponse);
    }

    private static String toStringValidationErrors(OpenApiApi api, ApiScenarioData scenarioData) throws ApiException {
        List<ApiValidationError> validationErrors = api.getValidationErrors(scenarioData.getId());
        StringBuilder sb = new StringBuilder("Scenario contains errors:\n");
        for (var apiStrictClassData : validationErrors) {
            String source = apiStrictClassData.getSource().stream().collect(joining(", "));
            sb.append(api.translate(apiStrictClassData.getMessage()))
                    .append("\t - \t").append(source)
                    .append('\n');
        }
        return sb.toString();
    }

    private static ApiIdNameObject getCustomEndTypeProperty(OpenApiApi api, ApiSimulationRunConfiguration simulationData) throws ApiException {
        var optionRequest = new ApiOptionRequest();
        optionRequest.setRcId(simulationData.getId());
        optionRequest.setType(ApiOptionRequest.TypeEnum.TIME_TYPE_END);
        ApiOptionList endPeriodTypes = api.getExperimentOptions(0, 1000, optionRequest);

        return endPeriodTypes.getList().stream()
                .filter(t -> t.getId().equals("SelectedDates"))
                .findFirst().orElseThrow();
    }
}